/*********************************************************************
 *
 * File:         session_manage.c
 *
 * Description:  Session management module for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-15-96
 *
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "global.h"
#include "session_manage.h"

Session session;

int sendToApps(int app, short tag, char *data, int len)
{
  Message *msg;
  int size;
  char userid[20];
  char hostname[50];
  int i;

  if(data != NULL)
    {
      size = sizeof(Message) + len - 1;
      msg = malloc(size);
      if(msg == NULL)
	fprintf(stderr, "Engine: control message allocation error\n");
      memcpy(msg->data, data, len);
    }
  else
    {
      size = sizeof(Message);
      msg = malloc(size);
      if(msg == NULL)
	fprintf(stderr, "Engine: control message allocation error\n");
      msg->data[0] = 0;
    }

  msg->tag = tag;
  cuserid(userid);
  gethostname(hostname, 50);
  strcpy(msg->sender.user, userid);
  strcpy(msg->sender.host, hostname);
  msg->local = LOCAL;

  if(app < 0)
    {
      for(i=0; i<MAX_APPS; i++)
	{
	  if(session.appActive[i])
	    {
	      printf("Send message to application\n");
	      if(sendControlPacket(&session.appAddr[i], msg, size) < 0)
		{
		  free(msg);
		  return 0; 
		}
	    }
	}
    }
  else
    {
      if(session.appActive[app])
	{
	  printf("Send message to application\n");
	  if(sendControlPacket(&session.appAddr[app], msg, size) < 0)
	    {
	      free(msg);
	      return 0; 
	    }
	
	}
    }
    
  free(msg);
  return 1;
}


int newMember(Participant *member)
{
  MemberInfo info;
  struct hostent *hostinfo;

  hostinfo = gethostbyname(member->host);
  if(hostinfo == NULL)
    {
      printf("Member: unknown host %s\n", member->host);
      return 0;
    }

  session.lastMember++;
  session.numMembers++;
  session.memberActive[session.lastMember] = 1;
  strcpy(session.member[session.lastMember].user, member->user);
  strcpy(session.member[session.lastMember].host, member->host);

  bzero(&session.memberAddr[session.lastMember], sizeof(struct sockaddr_in));
  session.memberAddr[session.lastMember].sin_family = AF_INET;
  session.memberAddr[session.lastMember].sin_addr.s_addr = 
    ((struct in_addr *)hostinfo->h_addr)->s_addr;
  session.memberAddr[session.lastMember].sin_port = htons(DAEMON_PORT);

  info.handle = session.lastMember;
  memcpy(&info.part, member, sizeof(Participant));
  memcpy(&info.addr, &session.memberAddr[session.lastMember], 
	 sizeof(struct sockaddr_in));
  sendControlMessage(LOCAL, 0, indLocalSessionAddMember, 
		     (char *)&info, sizeof(MemberInfo));

  sendToApps(-1, indLocalSessionAddMember, (char *)&info, sizeof(MemberInfo));
  return 1;
}

int getManagerAddr(struct sockaddr_in *addr)
{
  long hostid;

  bzero(addr, sizeof(struct sockaddr_in));
  addr->sin_family = AF_INET;
  hostid = gethostid();
  addr->sin_addr.s_addr = hostid;
  addr->sin_port = htons(MANAGER_PORT);
  return 1;
}

int getUserData(int userNo, struct sockaddr_in *addr, Participant *part)
{
  if((session.memberActive[userNo]) && (userNo != session.localMember))
    {
      memcpy(addr, &session.memberAddr[userNo], sizeof(struct sockaddr_in));
      memcpy(part, &session.member[userNo], sizeof(Participant));
      return 1;
    }
  else
    return 0;
}

int getUserHandle(Participant *part)
{
  int i;

  for(i=0; i<MAX_MEMBERS; i++)
    {
      if(session.memberActive[i])
	if(   (strcmp(session.member[i].host, part->host) == 0)
	   && (strcmp(session.member[i].user, part->user) == 0))
	      return i;
    }

  return -1;
}



int sessionInit()
{
  int i;

  session.status = statusInit;
  session.lastMember = -1;
  session.numMembers = 0;
  session.localMember = -1;
  for(i=0; i<MAX_MEMBERS; i++)
    session.memberActive[i] = 0;

  for(i=0; i<MAX_APPS; i++)
    session.appActive[i] = 0;
  session.numApps = 0;
  session.lastApp = -1;
}


int sessionCreate(Participant *initiator)
{
  if(session.status == statusInit)
    {
      printf("Engine: session initiator %s@%s\n", initiator->user, initiator->host);
      session.status = statusCreated;
      newMember(initiator);
      session.localMember = 0;
      return 1;
    }
  else
    {
      return 0;
    }
}

int sessionAddMember(Participant *newmember)
{
  int current;

  if((session.status == statusCreated) || (session.status == statusActive))
    {
      if(session.lastMember < MAX_MEMBERS-1)
	{
	  if(getUserHandle(newmember) == -1)
	    {
	      printf("Engine: session member %s@%s\n", 
		     newmember->user, newmember->host);
	      session.status = statusActive;
	      newMember(newmember);
	      return 1;
	    }
	}
    }
  return 0;
}

void sessionDeleteMember(Participant *member)
{
  int i;

  if(session.status == statusActive)
    {
      for(i=0; i<MAX_MEMBERS; i++)
	{
	  if((session.memberActive[i]) 
	     && (!strcmp(member->user, session.member[i].user))
	     && (!strcmp(member->host, session.member[i].host)))
	    {
	      session.memberActive[i] = 0;
	      session.numMembers--;
	      sendControlMessage(LOCAL, 0, indLocalSessionDeleteMember, 
				 (char *)&i, sizeof(int));
	      sendToApps(-1, indLocalSessionDeleteMember, (char *)&i, sizeof(int));
	    }
	}
    }
}


int sessionAddApp(struct sockaddr_in *addr)
{
  MemberInfo info;
  int i;

  if(session.lastApp < MAX_APPS - 1)
    {
      printf("Engine: add application\n");
      session.lastApp++;
      session.numApps++;
      session.appActive[session.lastApp] = 1;
      session.appPort[session.lastApp] = ntohs(addr->sin_port);
      memcpy(&session.appAddr[session.lastApp], addr, 
	     sizeof(struct sockaddr_in)); 
      session.appAddr[session.lastApp].sin_addr.s_addr = gethostid();

      for(i=0; i<MAX_MEMBERS; i++)
	{
	  if(session.memberActive[i])
	    {
	      info.handle = i;
	      memcpy(&info.part, &session.member[i], sizeof(Participant));
	      memcpy(&info.addr, &session.memberAddr[i], 
		     sizeof(struct sockaddr_in));
	      sendToApps(session.lastApp, indLocalSessionAddMember,
			 (char *)&info, sizeof(MemberInfo));
	    }
	}

      sendToApps(session.lastApp, indLocalSessionLocalMember, 
		 (char *)&session.localMember, sizeof(int));

      return 1;
    }
  else
    return 0;
}


void sessionDeleteApp(int port)
{
  int i;

  for(i=0; i<MAX_APPS; i++)
    {
      if(session.appPort[i] == port)
	session.appActive[i] = 0;
    }
}

void sessionDesactivate()
{
  session.status = statusNonActive;
}

void sessionActivate()
{
  session.status = statusActive;
}

int sessionGetNoMembers()
{
  return session.numMembers;
}


void sessionGetMembers(Participant *parts)
{
  int i, no = 0;

  for(i=0; i<MAX_MEMBERS; i++)
    {
      if(session.memberActive[i])
	{
	  memcpy(parts, &session.member[i], sizeof(Participant));
	  parts++;
	}
    }
}

void sessionSetLocalMember()
{
  session.localMember = session.lastMember;
  sendToApps(-1, indLocalSessionLocalMember, 
	     (char *)&session.localMember, sizeof(int));
}











