/*********************************************************************
 *
 * File:         application.c
 *
 * Description:  NPAC Conferencing System API functions for applications 
 *               management 
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-22-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "common.h"

extern int errno;



int appEngineRequest(ncsApplication *app, short tag, char *data, int len)
{
  Message *msg;
  int size;

  size = sizeof(Message) + len - 1;
  msg = malloc(size);
  if(msg == NULL)
    fprintf(stderr, "Application: control message allocation error\n");
  msg->local = 1;
  msg->tag = tag;
  memcpy(msg->data, data, len);
  cuserid(msg->sender.user);
  gethostname(msg->sender.host, 50);

  if(sendPacket(app->ctrl_socket, &app->daemon_addr, msg, size) < 0)
    return 0; 

  return 1;
}

void appAddMember(ncsApplication *app, char *data)
{
  MemberInfo *newmember;

  newmember = (MemberInfo *)data;

  PRINTF("Application: member added %s@%s handle %d\n", newmember->part.user, newmember->part.host, newmember->handle);

  app->memberActive[newmember->handle] = 1;
  memcpy(&app->member[newmember->handle], newmember, sizeof(MemberInfo));
  app->member[newmember->handle].addr.sin_port = htons(app->port);

  app->numMembers++;
}

void appDeleteMember(ncsApplication *app, char *data)
{
  int *no;

  no = (int *)data;
  app->memberActive[*no] = 0;
  app->numMembers--;
}

void appProcessMessage(ncsApplication *app, void *data, int len)
{
  Message *msg;
  int size;
  InviteData *info;

  msg = (Message *)data;
  size = len - sizeof(Message) + 1;
  info = (InviteData *)msg->data;
  switch((LocalTag)msg->tag)
    {
    case indLocalSessionAddMember:
      PRINTF("Application: add member\n");
      appAddMember(app, msg->data);
      break;

    case indLocalSessionDeleteMember:
      PRINTF("Application: delete member\n");
      appDeleteMember(app, msg->data);
      break;

    case indLocalSessionLocalMember:
      PRINTF("Application: local member %d\n", *(int *)msg->data);
      app->localHandle = *(int *)msg->data;
      if(app->startingCallback != NULL)
	app->startingCallback();
      break;

    default:
      fprintf(stderr, "Application: wrong message tag %d\n", (LocalTag)msg->tag);
    }
}

ncsApplication *ncsAppAttach(int port)
{
  ncsApplication *app;
  long hostid;

  app = malloc(sizeof(ncsApplication));
  
  app->port = port;
  
  /* Initialize application data socket */
  if ((app->socket = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
      fprintf(stderr, "Application: can't open socket\n");
      free(app);
      return NULL;
    }
  
  bzero(&app->addr, sizeof(struct sockaddr_in));
  app->addr.sin_family = AF_INET;
  app->addr.sin_addr.s_addr = htonl(INADDR_ANY);
  app->addr.sin_port        = htons(port);
  
  if(bind(app->socket, &app->addr, sizeof(struct sockaddr_in)) < 0)
    {
      fprintf(stderr, "Application: can't bind socket\n");
      free(app);
      return NULL;
    }

  /* Initialize application control socket */
  if ((app->ctrl_socket = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
      fprintf(stderr, "Application: can't open control socket\n");
      free(app);
      return NULL;
    }

  bzero(&app->ctrl_addr, sizeof(struct sockaddr_in));
  app->ctrl_addr.sin_family = AF_INET;
  app->ctrl_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  app->ctrl_addr.sin_port        = htons(port+1);
  
  if(bind(app->ctrl_socket, &app->ctrl_addr, sizeof(struct sockaddr_in)) < 0)
    {
      fprintf(stderr, "Application: can't bind control socket\n");
      free(app);
      return NULL;
    }

  /* Set engine address */
  bzero(&app->daemon_addr, sizeof(struct sockaddr_in));
  app->daemon_addr.sin_family = AF_INET;
  hostid = gethostid();
  app->daemon_addr.sin_addr.s_addr = htonl(hostid);
  app->daemon_addr.sin_port        = htons(DAEMON_PORT);

  /* Send request to engine */
  appEngineRequest(app, reqLocalApplicationAttach, 
		   (char *)&app->ctrl_addr, sizeof(struct sockaddr_in));


  app->numMembers = 0;
  app->startingCallback = NULL;

  return app;
}

void ncsAppDettach(ncsApplication *app)
{
  if(app != NULL)
    {
      /* Send request to engine */
      appEngineRequest(app, reqLocalApplicationDettach, 
		       (char *)&app->port, sizeof(int));
      close(app->socket);
      close(app->ctrl_socket);
      free(app);
    }
}

int ncsAppSendData(ncsApplication *app, int mbr, int type, void *data, int len)
{
  ncsData *ncsdata;
  int size;
  int i;

  if(len == 0)
    {
      size = sizeof(ncsData);
      ncsdata = malloc(size);
      ncsdata->data[0] = 0;
    }
  else
    {
      size = len + sizeof(ncsData) - 1;
      ncsdata = malloc(size);
      memcpy(ncsdata->data, data, len); 
    }

  ncsdata->type = type;
  ncsdata->handle = app->localHandle;

  if(mbr < 0)
    {
      for(i=0; i<MAX_MEMBERS; i++)
	if(app->memberActive[i])
	  {
	    if(i != app->localHandle)
	      {
		PRINTF("Application: data sent to user %d, size %d\n", i, size);
		app->member[i].addr.sin_port = htons(app->port);
		sendPacket(app->socket, &app->member[i].addr, ncsdata, size);
	      }
	  }
    }
  else
    {
      if(app->memberActive[mbr])
	if(mbr != app->localHandle)
	  {
	    app->member[mbr].addr.sin_port = htons(app->port);
	    sendPacket(app->socket, &app->member[mbr].addr, ncsdata, size);
	  }
    }      

  free(ncsdata);
}

int ncsAppProcessIncoming(ncsApplication *app, fd_set *set)
{
  int msg_len, n, remote_addr_len;
  struct sockaddr_in remote_addr;
  void *data;

  remote_addr_len = sizeof(struct sockaddr_in);
  if(FD_ISSET(app->ctrl_socket, set))
    {
      data = malloc(MAX_UDP_SIZE);
      n = recvfrom(app->ctrl_socket, data, MAX_UDP_SIZE, 
		   0, &remote_addr, &remote_addr_len); 
      if (n < 0)
	{
	  fprintf(stderr, "Application: recvfrom error\n");
	  return 0;
	}
      else
	{
	  printf("Application: received message size %d\n", n);
	  appProcessMessage(app, data, n); 
	}

      free(data);
    }
  else
    return 0;

  return 1;
}

int ncsAppGetData(ncsApplication *app, fd_set *set, char **data)
{
  int msg_len, n, remote_addr_len;
  struct sockaddr_in remote_addr;

  remote_addr_len = sizeof(struct sockaddr_in);
  if(FD_ISSET(app->socket, set))
    {
      PRINTF("Application: there are data\n");

      *data = malloc(MAX_UDP_SIZE);
      n = recvfrom(app->socket, (char *)*data, MAX_UDP_SIZE, 
		   0, &remote_addr, &remote_addr_len); 
      if (n < 0)
	{
	  fprintf(stderr, "Application: recvfrom error\n", errno);
	  return 0;
	}
    }
  else
    return 0;

  return n;
}  


int ncsAppGetFD(ncsApplication *app, fd_set *set)
{
  FD_ZERO(set);
  FD_SET(app->socket, set);
  FD_SET(app->ctrl_socket, set);
  return app->ctrl_socket;
}

int ncsAppGetLocalHandle(ncsApplication *app)
{
  return app->localHandle;
}

int ncsAppGetNumMembers(ncsApplication *app)
{
  return app->numMembers;
}

void ncsAppAddCallback(ncsApplication *app, short tag, void (* function))
{
  switch(tag)
    {
    case tagAppStarting:
      app->startingCallback = function;
      break;

    default:
      fprintf(stderr, "Unknown application event tag\n");
    }
}
	


