/*********************************************************************
 *
 * File:         sessions.c
 *
 * Description:  NPAC Conferencing System API functions for session 
 *               management 
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "common.h"

MemberInfo member[MAX_MEMBERS];
int memberActive[MAX_MEMBERS];
int localHandle;

void ncsEngineMessageHandle(void *message, int len);


void (* inviteCallback)(void *);
void (* deleteMemberCallback)(void *);
void (* addMemberCallback)(void *);

ManagerData managerData;


void InitMembers()
{
  int i;

  for(i=0; i<MAX_MEMBERS; i++)
    memberActive[i] = 0;
}

void AddMember(char *data)
{
  MemberInfo *newmember;

  newmember = (MemberInfo *)data;
  printf("Member added locally %s@%s handle %d\n", newmember->part.user, newmember->part.host, newmember->handle);
  memberActive[newmember->handle] = 1;
  memcpy(&member[newmember->handle], newmember, sizeof(MemberInfo));

  addMemberCallback(&newmember->part);
}

void DeleteMember(char *data)
{
  int *no;

  no = (int *)data;
  memberActive[*no] = 0;

  deleteMemberCallback(&member[*no].part);
}

void printMembers(InviteData *data, int len)
{
  int no, i;

  no = data->numMembers;
  printf("Conference members:\n");
  for(i=0; i<no; i++)
    printf("%s@%s\n", data->parts[i].user, data->parts[i].host);
}

void ncsSessionInitialize()
{
  long hostid;

  if ((managerData.socket = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    fprintf(stderr, "Manager: can't open manage socket\n");

  bzero(&managerData.addr, sizeof(managerData.addr));
  managerData.addr.sin_family = AF_INET;
  managerData.addr.sin_addr.s_addr = htonl(INADDR_ANY);
  managerData.addr.sin_port        = htons(MANAGER_PORT);

  if(bind(managerData.socket, &managerData.addr, sizeof(managerData.addr)) < 0)
    fprintf(stderr, "Manager: can't bind daemon socket\n");

  bzero(&managerData.daemon_addr, sizeof(managerData.daemon_addr));
  managerData.daemon_addr.sin_family = AF_INET;
  hostid = gethostid();
  managerData.daemon_addr.sin_addr.s_addr = htonl(hostid);
  managerData.daemon_addr.sin_port        = htons(DAEMON_PORT);

  ncsEngineRequest(reqLocalSessionInit, NULL, 0);

  InitMembers();
}

void ncsSessionCreate()
{
  ncsEngineRequest(reqLocalSessionCreate, NULL, 0);
  localHandle = 0;     /* what if session already created?!!! */
}


/* Registers working procedure running in the Xt background, 
   processing incoming control messages */
void ncsSessionRegisterWithXt(/*XtApp app*/)
{
}

/* Returns file descriptor to session control socket */
int ncsSessionGetFD(fd_set *set)
{
  FD_ZERO(set);
  FD_SET(managerData.socket, set);
  return managerData.socket;
}

/* Processes control messages coming to the control socket */
void ncsProcessIncoming(fd_set *set)
{
  void *msg;
  int n, msg_len;
  struct sockaddr_in addr;
  int addr_size;
  
  addr_size = sizeof(addr);
  if(FD_ISSET(managerData.socket, set))
    {
      msg = malloc(MAX_UDP_SIZE);
      n = recvfrom(managerData.socket, msg, MAX_UDP_SIZE, 0, &addr, &addr_size); 
      if (n < 0)
	fprintf(stderr, "Manager: recvfrom error\n");
      
      ncsEngineMessageHandle(msg, n);
      
      free(msg);
    }
}


void ncsEngineMessageHandle(void *message, int len)
{
  Message *msg;
  InviteData *info;
  int size;

  msg = (Message *)message;
  size = len - sizeof(Message) + 1;
  info = (InviteData *)msg->data;
  switch((LocalTag)msg->tag)
    {
    case indLocalSessionInvited:
      inviteCallback((Participant *)msg->data);
      break;

    case indLocalSessionAccept:
      printf("Manager: Invitation accepted\n");
      break;

    case indLocalSessionReject:
      printf("Manager: Invitation rejected\n");
      break;
      
    case indLocalSessionMembers:
      printMembers(info, size);
      break;

    case indLocalSessionAddMember:
      printf("Manager: add member\n");
      AddMember(msg->data);
      break;

    case indLocalSessionDeleteMember:
      DeleteMember(msg->data);
      break;

    case indLocalSessionLocalMember:
      localHandle = *(int *)msg->data;
      break;

    default:
      fprintf(stderr, "Manager: wrong message tag %d\n", (LocalTag)msg->tag);
    }
}

void ncsSessionAddMember(Participant *part)
{
  ncsEngineRequest(reqLocalSessionAddMember, part, sizeof(Participant));
}

void ncsSessionAcceptInvitation(Participant *part)
{
  printf("Manager: join session %s@%s \n", part->user, part->host);
  ncsEngineRequest(reqLocalSessionJoin, part, sizeof(Participant));
}

void ncsSessionRejectInvitation()
{
}

void ncsSessionAddCallback(short tag, void (*function))
{
  switch(tag)
    {
    case indLocalSessionInvited:
      inviteCallback = function;
      break;

    case indLocalSessionAddMember:
      addMemberCallback = function;
      break;

    case indLocalSessionDeleteMember:
      deleteMemberCallback = function;
      break;

    default:
      fprintf(stderr, "Manager: Unknown control callback type\n");
    }
}

void ncsSessionQuit()
{
  ncsEngineRequest(reqLocalSessionQuit, NULL, 0);
  close(managerData.socket);
}

void ncsSessionQueryMembers()
{
  ncsEngineRequest(reqLocalSessionGetMembers, NULL, 0);
}





